#!/usr/bin/python

import sys
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL import GLX
from OpenGL.raw._GLX import struct__XDisplay
from OpenGL import GL
from ctypes import *

import Xlib
from Xlib.display import Display
from gi.repository import Gtk, GdkX11, Gdk

from cad import point
from cad import line
from cad import plane
from cad import camera
from libs.mouse_handler import mouse_state


class gtkgl:
    """ These method do not seem to exist in python x11 library lets exploit the c methods """
    xlib = cdll.LoadLibrary('libX11.so')
    xlib.XOpenDisplay.argtypes = [c_char_p]
    xlib.XOpenDisplay.restype = POINTER(struct__XDisplay)
    xdisplay = xlib.XOpenDisplay("")
    display = Xlib.display.Display()
    attrs = []

    xwindow_id = None
    width = height = 200

    def __init__(self):
        """ Lets setup are opengl settings and create the context for our window """

        self.add_attribute(GLX.GLX_RGBA, True)
        self.add_attribute(GLX.GLX_RED_SIZE, 1)
        self.add_attribute(GLX.GLX_GREEN_SIZE, 1)
        self.add_attribute(GLX.GLX_BLUE_SIZE, 1)
        self.add_attribute(GLX.GLX_DOUBLEBUFFER, 0)

        xvinfo = GLX.glXChooseVisual(self.xdisplay, self.display.get_default_screen(), self.get_attributes())
        configs = GLX.glXChooseFBConfig(self.xdisplay, 0, None, byref(c_int()))
        self.context = GLX.glXCreateContext(self.xdisplay, xvinfo, None, True)

    def add_attribute(self, setting, value):
        """a simple method to nicely add opengl parameters"""
        self.attrs.append(setting)
        self.attrs.append(value)

    def get_attributes(self):
        """ return our parameters in the expected structure"""
        attrs = self.attrs + [0, 0]
        return (c_int * len(attrs))(*attrs)

    def configure(self, wid):
        """ setup up a opengl viewport for the current window, grab the xwindow id and store for later usage"""
        self.xwindow_id = GdkX11.X11Window.get_xid(wid)
        if(not GLX.glXMakeCurrent(self.xdisplay, self.xwindow_id, self.context)):
            print ('configure failed running glXMakeCurrent')
        glViewport(0, 0, self.width, self.height)

    def draw_start(self):
        """make cairo context current for drawing"""
        if(not GLX.glXMakeCurrent(self.xdisplay, self.xwindow_id, self.context)):
            print ('draw failed running glXMakeCurrent')

    def draw_finish(self):
        """swap buffer when we have finished drawing"""
        GLX.glXSwapBuffers(self.xdisplay, self.xwindow_id)

    def test(self):
        """Test method to draw something so we can make sure opengl is working and we can see something"""
        self.draw_start()

        glClearColor(0.0, 0.0, 0.0, 0.0)
        glClear(GL_COLOR_BUFFER_BIT)
        glBegin(GL_TRIANGLES)
        glIndexi(0)
        glColor3f(1.0, 0.0, 0.0)
        glVertex2i(0, 1)
        glIndexi(0)
        glColor3f(0.0, 1.0, 0.0)
        glVertex2i(-1, -1)
        glIndexi(0)
        glColor3f(0.0, 0.0, 1.0)
        glVertex2i(1, -1)
        glEnd()

        self.draw_finish()


class gui:
    """cad drawing application."""
    glwrap = gtkgl()

    def __init__(self):
        """ Initialise the GTK interface grab the GTK widgets, connect the events we are interested in receiving"""
        xml = Gtk.Builder()
        xml.add_from_file('interface.glade')
        self.window = xml.get_object('window')
        self.window.connect('delete_event', Gtk.main_quit)
        self.window.connect('destroy', lambda quit: Gtk.main_quit())
        self.window.set_reallocate_redraws(True)

        self.drawing_area = xml.get_object('drawingarea')
        self.drawing_area.set_double_buffered(False)
        self.drawing_area.set_size_request(self.glwrap.width, self.glwrap.height)

        #add mouse click events to the drawing area
        self.drawing_area.add_events(Gdk.EventMask.BUTTON_PRESS_MASK)

        self.drawing_area.connect('configure_event', self.on_configure_event)
        self.drawing_area.connect('draw', self.on_draw)
        self.drawing_area.connect('realize', self.on_draw)
        
        #hook up the mouse click to a method
        self.drawing_area.connect("button_press_event", self.mouse_click)

        #initialise the camera with our new gtk widget width and height
        self.camera = camera.createcamera(self.glwrap.width, self.glwrap.height)
        self.mouse = mouse_state()
        
        self.test_point = point.createpoint((10, 10, -20))
        self.test_lines = []
        
        self.test_grid_plane = plane.createplanesimple((0, 0, 50), 80)
        self.test_plane = plane.createplane((0, 0, 30), (20, 0, 30), (0, 20, 30))
        
        self.window.show_all()

    def on_configure_event(self, widget, event):
        self.glwrap.configure(widget.get_window())
        return True

    def mouse_click(self, widget, event):
        self.mouse.append(event.x, event.y, event.button)
        self.test_point = self.camera.get_click_point(self.mouse.get_click_position())
        
        #new test for our line code to make sure its functioning correctly
        if self.mouse.count() == 2:
            p1, p2 = self.mouse.get_points()
            tp1 = self.camera.get_click_point(p1)
            tp2 = self.camera.get_click_point(p2)
            self.test_lines.append(line.createline([tp1.x, tp1.y, tp1.z], [tp2.x, tp2.y, tp2.z]))
        self.on_draw()

    def on_draw(self, *args):
        """ Test code to make sure we can draw a pixel successfully,
        also test we can position our new camera class to lookat the pixel"""
        #lets not look directly at the point we are drawing, demonstrating we can lookat points in space

        #recalculate our camera based on the new settings
        self.camera.update()

        glClearColor(0.0, 0.0, 0.0, 0.0)
        glClear(GL_COLOR_BUFFER_BIT)
        self.glwrap.draw_start()

        self.test_grid_plane.draw()
        self.test_plane.draw()

        #place a point we can lookat that will be positioned in our field of view.
        self.test_point.draw()

        #draw our test lines
        for line in self.test_lines:
            line.draw()

        self.glwrap.draw_finish()

    def quit(self, *args):
        Gtk.main_quit()


g = gui()
Gtk.main()
